      PROGRAM GLC
C
C     THIS PROGRAMME IS DESIGNED TO TAKE IN A SEQUENCE OF
C     INCOMES, ORDER THESE INCOMES, CREATE THE GENERALIZED
C     LORENZ CURVE (GLC) OF THE INCOMES, CALCULATE AN EST-
C     IMATE OF THE VARIANCE-COVARIANCE MATRIX OF THE GLC
C     AND THEN OUTPUT THE GLC AND ITS STANDARD ERRORS FOR 
C     PLOTTING.  NOTE THE NUMBER OF ORDINATES CAN BE CHOSEN
C     BY THE USER AND IS LISTED AS THE PARAMETER  K.  FOR
C     EXAMPLE, TO GET DECILES CHOOSE K=9; TO GET QUINTILES
C     CHOOSE K=4.  THE ESTIMATE OF THE VARIANCE-COVARIANCE
C     MATRIX IS AN IMPLEMENTATION OF THE METHODS PROPOSED 
C     BY BISHOP, CHAKRABORTI, AND THISTLE [1989].  THE FULL
C     VARIANCE-COVARIANCE CAN ALSO BE USED TO TEST WHETHER 
C     GLCS DIFFER.
C
C                       HARRY J. PAARSCH @10:44 11/VIII/94
C
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(N=1000,K=9)
      CHARACTER*8 INPUT
      INTEGER*4 ITAG(0:K+1),ICNT(0:K+1)
      REAL*8 Y(N)
      REAL*8 DLAMDA(0:K+1),GAMMA(0:K+1),P(0:K+1),PHI(0:K+1),
     &       PSI(0:K+1),V(1:K+1,1:K+1),XI(0:K+1)
 
C 
C     INPUT THE DATA (Y) IN FREE FORMAT FROM A FILE 
C     CALLED "DATA".
C
 
      INPUT = 'test.dat'
      OPEN(UNIT=5,FILE=INPUT)
      DO 1 I=1,N
        READ(5,*) Y(I)
C       WRITE(6,11) Y(I)
   11   FORMAT(1X,F7.4)
    1 CONTINUE
 
C
C     SORT THE DATA USING A BUBBLE SORT WHICH IS PROVIDED.
C     IF THE DATA SET IS LARGE, THEN THE USER MAY WANT TO 
C     USE ANOTHER SORTING ALGORITHM, SUCH AS A HEAP SORT.
C
 
      CALL SORT(Y,N)
 
      DO 21 I=1,N
C       WRITE(6,11) Y(I)
   21 CONTINUE
 
C
C     INITIALIZE PROPORTIONS (P) KNOWN EX ANTE.  USING A
C     TAG (ITAG) VECTOR FIND THE PERCENTILES (XI) FOR EACH
C     OF THE CHOSEN PROPORTIONS.  NOTE, DUE TO ROUND OFF
C     ERROR IN THE MAKING OF INTEGERS FROM REALS SOME
C     OF THE CELLS MAY BE ONE COUNT OUT.  FOR EXAMPLE,
C     IF K=9 AND THE NUMBER OF OBSERVATIONS, N=1000
C     THEN THE FIRST CELL WILL HAVE 99 OBSERVATIONS.
C     ALL OTHERS WILL HAVE 100, BUT THE LAST WILL HAVE
C     101.  A COUNT VECTOR IS PROVIDED SO THE READER
C     CAN CHECK FOR THESE PATHOLOGIES AND CORRECT THEM
C     IN THE PROGRAMME MANUALLY.
C
 
      P(0)      = 0.D0
      P(K+1)    = 1.D0
      ITAG(0)   = 0
      ITAG(K+1) = N
      PROP = 1.D0/(K+1)
C     WRITE(6,*) PROP
      DO 2 J=1,K
        P(J) = PROP*J
        ITAG(J) = P(J)*N
    2 CONTINUE
 
      XI(0)   = 0.D0
      ICNT(0) = 0
      DO 3 J=1,K+1
        XI(J)   = Y(ITAG(J))
        ICNT(J) = ITAG(J) - ITAG(J-1)
    3 CONTINUE
 
C
C     CALCULATES THE MEANS WITHIN EACH PERCENTILE (GAMMA)
C     AND FIND THE GLC ORDINATES (PHI).
C
 
      GAMMA(0) = 0.D0
      DO 4 J=1,K+1
        GAMMA(J) = 0.D0
        DO 5 I=1,ITAG(J)
          GAMMA(J) = GAMMA(J) + (Y(I)/ITAG(J))
    5   CONTINUE
    4 CONTINUE
 
      PHI(0) = 0.D0
      DO 6 J=1,K+1
        PHI(J) = GAMMA(J)*P(J)
    6 CONTINUE
 
C
C     FIND THE VARIANCES (DLAMDA) WITHIN THE PERCENTILE
C     CELLS AS WELL AS THE INCOME SHARES (PSI).
C
 
      DLAMDA(0) = 0.D0
      PSI(0)    = 0.D0
      DO 7 J=1,K+1
        DLAMDA(J) = 0.D0
        PSI(J) = PHI(J) - PHI(J-1)
        DO 8 I=1,ITAG(J)
          DLAMDA(J) = DLAMDA(J) + (Y(I)-GAMMA(J))*(Y(I)-GAMMA(J))
    8   CONTINUE
        DLAMDA(J) = DLAMDA(J)/ITAG(J)
    7 CONTINUE
 
C
C     CALCULATE THE VARIANCE-COVARIANCE MATRIX OF THE GLC (V).
C
 
      DO 31 J=1,K+1
        DO 32 I=1,J
          V(I,J) = P(I)*(DLAMDA(I) +
     &             (1-P(J))*(XI(I)-GAMMA(I))*(XI(J)-GAMMA(J)) +
     &             (XI(I)-GAMMA(I))*(GAMMA(J)-GAMMA(I)))
          V(J,I) = V(I,J)
   32   CONTINUE
   31 CONTINUE
 
C
C     OUTPUT THE GLC AND ITS STANDARD ERRORS FOR PLOTTING PURPOSES.
C
 
      WRITE(6,92)
   92 FORMAT(/,/,19X,'THE GLC AND ITS STANDARD ERRORS')
      WRITE(6,93) N
   93 FORMAT(    16X,'           SAMPLE SIZE = ',I4,/)
      WRITE(6,94)
   94 FORMAT(/,19X,' J      P(J)    PHI(J)     SE(J)',/)
      DO 51 J=1,K+1
        WRITE(6,52) J,P(J),PHI(J),DSQRT(V(J,J)/N)
   52   FORMAT(19X,I2,3(F10.5))
   51 CONTINUE
 
      DO 61 J=1,K+1
C       WRITE(6,62) PHI(J),(V(I,J),I=1,K+1)
   62   FORMAT(11(F7.4))
   61 CONTINUE
 
      STOP
      END
 
      SUBROUTINE SORT(Y,N)
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 Y(N)
 
      DO 1 I=1,N-1
        DO 2 J=I,N
          IF (Y(I) .GT. Y(J)) THEN
            DUMMY = Y(I)
            Y(I) = Y(J)
            Y(J) = DUMMY
          END IF
    2   CONTINUE
    1 CONTINUE
 
      RETURN
      END
